@@ -41,8 +41,47 @@ setMethod("iteration", "numeric",
 )
 
 
+#setMethod("iteration", "simObj",
+#  function(y, times=NULL, func=NULL, parms=NULL, animate=FALSE, ...) {
+#    init              <- y@init
+#    times             <- fromtoby(y@times)
+#    func              <- y@main
+#    parms             <- y@parms
+#    inputs            <- y@inputs
+#    equations         <- y@equations
+#    environment(func) <- environment()
+#    equations               <- addtoenv(equations)
+#    parms$DELTAT <- 0
+#    out <- list(func(times[1], init, parms))
+#    for (i in 2:length(times)) {
+#      time <- times[i]
+#      parms$DELTAT <- times[i] - times[i-1]
+#      init <- func(time, init, parms)
+#      out  <- c(out, list(init))
+#      if (animate) {
+#        y@out   <- out
+#        plot(y, index=i, ...)
+#      }
+#    }
+#    out
+#  }
+#)
+
 setMethod("iteration", "simObj",
   function(y, times=NULL, func=NULL, parms=NULL, animate=FALSE, ...) {
+    observer = function(init, time, i, out, y){
+      if (is.null(y@observer)) {
+        ## default: simply return the state
+        init
+      } else {
+        ## call a function provided by the observer slot of the simObj
+        if (length(formals(y@observer)) == 1) {
+          y@observer(init)                    # for compatibility
+        } else {
+          y@observer(init, time, i, out, y)   # experimental
+        }
+      }
+    }
     init              <- y@init
     times             <- fromtoby(y@times)
     func              <- y@main
@@ -50,20 +89,38 @@ setMethod("iteration", "simObj",
     inputs            <- y@inputs
     equations         <- y@equations
     environment(func) <- environment()
-    equations               <- addtoenv(equations)
+    equations         <- addtoenv(equations)
     parms$DELTAT <- 0
-    out <- list(func(times[1], init, parms))
+    res <- observer(init, times[1], 1, NULL, y)
+    if (is.vector(res)) {
+        out  <- res
+      } else {
+        out  <- list(res)
+      }
     for (i in 2:length(times)) {
       time <- times[i]
       parms$DELTAT <- times[i] - times[i-1]
       init <- func(time, init, parms)
-      out  <- c(out, list(init))
+      res  <- observer(init, time, i, out, y)
+      if (is.vector(res)) {
+        out  <- rbind(out, res, deparse.level=0)
+      } else {
+        out  <- c(out, list(res))
+      }
+      ## animate is deprecated and may be removed in future version
+      ##   use the observer mechanism instead
       if (animate) {
-        y@out   <- out
-        plot(y, index=i, ...)
+         y@out   <- out
+         plot(y, index=i, ...)
       }
     }
-    out
+    if(is.vector(res)) {
+      # row.names(out) <- NULL ## now obsolete, see deparse.level=0
+      out <- cbind(times, out)
+      out <- as.data.frame(out)
+    } else {
+      out
+    }
   }
 )
 
